package com.tzld.longarticle.recommend.server.service;

import cn.hutool.core.io.resource.ResourceUtil;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.repository.crawler.AccountAvgInfoRepository;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.AccountAvgInfo;
import com.tzld.longarticle.recommend.server.service.score.AvgReadDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

@Component
@Slf4j
public class AccountIndexAvgViewCountService {

    @Autowired
    AccountAvgInfoRepository accountAvgInfoRepository;

    private JSONObject accountInfo;

    @PostConstruct
    public void init() {
        String cardJSON = ResourceUtil.readUtf8Str("file/AccountInfo.json");
        accountInfo = JSONObject.parseObject(cardJSON);
    }


    public AvgReadDTO getAvgReadDto(String key) {
        return accountInfo.getObject(key, AvgReadDTO.class);
    }

    public double getAvgReadCount(String ghId, Integer index) {
        AvgReadDTO dto = getAvgReadDto(ghId + "_" + index);
        if (Objects.isNull(dto)) {
            dto = getAvgReadDto(ghId + "_4");
            if (Objects.isNull(dto)) {
                dto = getAvgReadDto(ghId + "_1");
            }
        }
        return dto == null ? 1.0 : dto.getReadAvg();
    }

    public double getAvgReadCountByDB(String ghId, Integer index) {
        List<AccountAvgInfo> accountAvgInfoList = accountAvgInfoRepository.getAllByGhIdEqualsAndStatusEquals(ghId, 1);
        if (CollectionUtils.isEmpty(accountAvgInfoList)) {
            return 1.0;
        }
        Map<String, AccountAvgInfo> accountAvgInfoMap = accountAvgInfoList.stream().collect(Collectors.toMap(AccountAvgInfo::getPosition, Function.identity()));
        AccountAvgInfo info = accountAvgInfoMap.get(index + "");
        if (Objects.isNull(info)) {
            info =  accountAvgInfoMap.get("4");
            if (Objects.isNull(info)) {
                info =  accountAvgInfoMap.get("1");
            }
        }
        return info == null ? 1.0 : info.getReadAvg();
    }

    public double getAvgReadCountByDB(List<AccountAvgInfo> accountAvgInfoListParam, String ghId, Integer index) {
        if (CollectionUtils.isEmpty(accountAvgInfoListParam)) {
            return 1.0;
        }
        Map<String, List<AccountAvgInfo>> accountAvgInfoAccountMap = accountAvgInfoListParam.stream().collect(Collectors.groupingBy(AccountAvgInfo::getGhId));
        List<AccountAvgInfo> accountAvgInfoList = accountAvgInfoAccountMap.get(ghId);
        if (CollectionUtils.isEmpty(accountAvgInfoList)) {
            return 1.0;
        }
        Map<String, AccountAvgInfo> accountAvgInfoMap = accountAvgInfoList.stream().collect(Collectors.toMap(AccountAvgInfo::getPosition, Function.identity()));
        AccountAvgInfo info = accountAvgInfoMap.get(index + "");
        if (Objects.isNull(info)) {
            info =  accountAvgInfoMap.get("4");
            if (Objects.isNull(info)) {
                info =  accountAvgInfoMap.get("1");
            }
        }
        return info == null ? 1.0 : info.getReadAvg();
    }

}
