package com.tzld.longarticle.recommend.server.service.rank.strategy;


import com.tzld.longarticle.recommend.server.common.enums.ContentPoolEnum;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.rank.RankItem;
import com.tzld.longarticle.recommend.server.service.rank.RankParam;
import com.tzld.longarticle.recommend.server.service.rank.RankResult;
import com.tzld.longarticle.recommend.server.service.rank.RankStrategy;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreResult;
import com.tzld.longarticle.recommend.server.service.score.ScoreService;
import com.tzld.longarticle.recommend.server.service.score.strategy.*;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author dyp
 */
@Service
@Slf4j
public class FwhColdStartRankStrategy implements RankStrategy {

    @Autowired
    private ScoreService scoreService;
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    public RankResult rank(RankParam param) {

        ScoreResult scoreResult = scoreService.score(convertToScoreParam(param));

        Map<String, Map<String, Double>> scoreMap = scoreResult.getScoreMap();
        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());

        List<RankItem> items = CommonCollectionUtils.toList(param.getContents(), c -> {
            RankItem item = new RankItem();
            c.setHisPublishArticleList(null);
            item.setContent(c);
            item.setScoreMap(scoreMap.get(c.getId()));
            double score;
            if (contentPools[0].equals(item.getContent().getContentPoolType())
                    || contentPools[1].equals(item.getContent().getContentPoolType())) {
                score = item.getScore(SimilarityStrategy.class.getSimpleName())
                        + item.getScore(CategoryStrategy.class.getSimpleName())
                        + item.getScore(FlowCtlDecreaseStrategy.class.getSimpleName());
                if (item.getScore(PublishTimesStrategy.class.getSimpleName()) >= 0) {
                    score += item.getScore(ViewCountRateStrategy.class.getSimpleName());
                }
            } else {
                score = item.getScore(SimilarityStrategy.class.getSimpleName())
                        + item.getScore(CategoryStrategy.class.getSimpleName())
                        + item.getScore(AccountPreDistributeStrategy.class.getSimpleName())
                        + item.getScore(PublishTimesStrategy.class.getSimpleName())
                        + item.getScore(FlowCtlDecreaseStrategy.class.getSimpleName());
            }
            item.setScore(score);
            return item;
        });

        // 1 排序
        Collections.sort(items, (o1, o2) -> -Double.compare(o1.getScore(), o2.getScore()));
        // 2 相似去重
        List<Content> contents = CommonCollectionUtils.toList(items, RankItem::getContent);
//        contents = deduplication(contents);

        // 3 选文章
        List<Content> result = new ArrayList<>();
        int size = param.getSize() * param.getUserGroupIds().size();
        result.addAll(contents.subList(0, Math.min(contents.size(), size)));

        return new RankResult(result);
    }

    private ScoreParam convertToScoreParam(RankParam param) {
        ScoreParam scoreParam = new ScoreParam();
        scoreParam.setGhId(param.getGhId());
        scoreParam.setAccountName(param.getAccountName());
        scoreParam.setContents(param.getContents());
        scoreParam.setStrategy(param.getStrategy());
        return scoreParam;
    }

    private List<Content> deduplication(List<Content> contents) {
        List<String> titles = new ArrayList<>();
        List<Content> result = new ArrayList<>();
        // 遍历所有列表
        for (String contentPool : ContentPoolEnum.getOrderContentPool()) {
            for (Content c : contents) {
                if (!contentPool.equals(c.getContentPoolType())) {
                    continue;
                }
                if (!TitleSimilarCheckUtil.isDuplicateContent(c.getTitle(), titles)) {
                    result.add(c);
                    titles.add(c.getTitle());
                }
            }
        }

        return result;
    }

}
