package com.tzld.longarticle.recommend.server.service.score.strategy;

import cn.hutool.core.collection.CollectionUtil;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.repository.aigc.CrawlerMetaArticleRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.PublishSortLogRepository;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.PublishSortLog;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.score.AccountPublishTimesConfig;
import com.tzld.longarticle.recommend.server.service.score.Score;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import java.util.function.Function;


import java.util.*;
import java.util.stream.Collectors;

@Component
@Slf4j
public class PublishTimesStrategy implements ScoreStrategy {

    @Autowired
    CrawlerMetaArticleRepository crawlerMetaArticleRepository;
    @Autowired
    AccountContentPoolConfigService accountContentPoolConfigService;
    @Autowired
    private PublishSortLogRepository publishSortLogRepository;

    @ApolloJsonValue("${accountPublishTimesConfig:{}}")
    private Map<String, AccountPublishTimesConfig[]> accountPublishTimesConfigMap;


    @Override
    public List<Score> score(ScoreParam param) {
        long start = System.currentTimeMillis();
        List<Score> scores = new ArrayList<>();
        if (CollectionUtils.isEmpty(param.getContents())) {
            return scores;
        }
        AccountPublishTimesConfig[] indexPublishTimesArr = getPublishTimes(param.getAccountName());
        if (Objects.isNull(indexPublishTimesArr) || indexPublishTimesArr.length == 0) {
            return scores;
        }
        // 获取今日已发布内容
        String dateStr = DateUtils.getCurrentDateStr("yyyy-MM-dd");
        List<PublishSortLog> hisPublishContentList = publishSortLogRepository.findByDateStr(dateStr);
        Map<String, Set<PublishSortLog>> hisPublishedContentMap = hisPublishContentList.stream()
                .collect(Collectors.groupingBy(
                        PublishSortLog::getTitle,
                        Collectors.collectingAndThen(
                                Collectors.toMap(
                                        PublishSortLog::getGhId, // 用于去重的字段，例如 id
                                        Function.identity(),
                                        (existing, replacement) -> existing // 如果有重复的 id，保留已有的记录
                                ),
                                map -> new HashSet<>(map.values()) // 将结果转换为 Set<PublishSortLog>
                        )
                ));
        Map<Integer, AccountPublishTimesConfig> indexPublishTimesMap = Arrays.stream(indexPublishTimesArr).collect(Collectors.toMap(AccountPublishTimesConfig::getIndex, o -> o));
        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        for (Content content : param.getContents()) {
            Score score = new Score();
            score.setStrategy(this);
            score.setContentId(content.getId());
            score.setScore(0.0);
            for (int i = 0; i < contentPools.length; i++) {
                int val = i + 1;
                if (CollectionUtils.isEmpty(hisPublishedContentMap.get(content.getTitle()))
                        || Objects.isNull(indexPublishTimesMap.get(val))) {
                    break;
                }
                List<PublishSortLog> filterArticles = hisPublishedContentMap.get(content.getTitle()).stream()
                        .filter(o -> (val == 1 && o.getIndex() == 1) || (val == 2 && o.getIndex() == 2) || (val == 3 && o.getIndex() >= 3))
                        .collect(Collectors.toList());
                if (CollectionUtil.isEmpty(filterArticles)) {
                    break;
                }
                if (contentPools[i].equals(content.getContentPoolType())) {
                    if (filterArticles.size() >= indexPublishTimesMap.get(val).getTimes()) {
                        score.setScore(score.getScore() + indexPublishTimesMap.get(val).getWeight());
                        break;
                    }
                }
            }
            scores.add(score);
        }
        log.info("PublishTimesStrategy cost:{}", System.currentTimeMillis() - start);
        return scores;
    }

    public AccountPublishTimesConfig[] getPublishTimes(String accountName) {
        if (accountPublishTimesConfigMap.containsKey(accountName)) {
            return accountPublishTimesConfigMap.get(accountName);
        }
        return accountPublishTimesConfigMap.get("default");
    }

}
