package com.tzld.longarticle.recommend.server.service.score.strategy;

import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.repository.adplatform.ChangwenArticleDatastatRepository;
import com.tzld.longarticle.recommend.server.repository.adplatform.ChangwenArticleRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.AccountAvgInfoRepository;
import com.tzld.longarticle.recommend.server.repository.entity.adplatform.ChangwenArticle;
import com.tzld.longarticle.recommend.server.repository.entity.adplatform.ChangwenArticleDatastat;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.AccountAvgInfo;
import com.tzld.longarticle.recommend.server.service.AccountIndexAvgViewCountService;
import com.tzld.longarticle.recommend.server.service.score.Score;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.NormalizationUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Component
@Slf4j
public class ViewMultiplierStrategy implements ScoreStrategy {

    @Autowired
    ChangwenArticleRepository changwenArticleRepository;
    @Autowired
    ChangwenArticleDatastatRepository changwenArticleDatastatRepository;
    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;
    @Autowired
    AccountAvgInfoRepository accountAvgInfoRepository;

    @Override
    public List<Score> score(ScoreParam param) {
        long start = System.currentTimeMillis();

        if (CollectionUtils.isEmpty(param.getContents())) {
            return Collections.emptyList();
        }
        List<String> channelContentIds = param.getContents().stream().map(Content::getCrawlerChannelContentId).collect(Collectors.toList());
        Map<String, ChangwenArticle> changwenArticleDTOMap = getContentIndex(channelContentIds);
        Map<String, ChangwenArticleDatastat> changwenArticleDatastatDTOMap = getContentViewCount(channelContentIds);

        List<AccountAvgInfo> avgInfoList = accountAvgInfoRepository.getAllByStatusEquals(1);
        List<Score> scores = CommonCollectionUtils.toList(param.getContents(), c -> {
            Score score = new Score();
            score.setContentId(c.getId());
            ChangwenArticle article = changwenArticleDTOMap.get(c.getCrawlerChannelContentId());
            ChangwenArticleDatastat datastat = changwenArticleDatastatDTOMap.get(c.getCrawlerChannelContentId());

            if (Objects.nonNull(article) && Objects.nonNull(datastat)) {
                double avgReadCount = accountIndexAvgViewCountService.getAvgReadCountByDB(avgInfoList, article.getAccountId(), article.getItemIndex());
                Integer readCount = datastat.getReadCount();
                score.setScore(NormalizationUtils.min(Math.max(readCount, 0) / avgReadCount - 1));
            } else {
                score.setScore(-1.0);
            }
            score.setStrategy(this);
            return score;
        });
        log.info("ViewMultiplierStrategy cost:{}", System.currentTimeMillis() - start);
        return scores;
    }

    private Map<String, ChangwenArticle> getContentIndex(List<String> channelContentIds) {
        if (CollectionUtils.isEmpty(channelContentIds)) {
            return new HashMap<>();
        }
        List<ChangwenArticle> result = changwenArticleRepository.findAllById(channelContentIds);
        if (CollectionUtils.isEmpty(result)) {
            return new HashMap<>();
        }
        return result.stream().collect(Collectors.toMap(ChangwenArticle::getId, o -> o));
    }

    private Map<String, ChangwenArticleDatastat> getContentViewCount(List<String> channelContentIds) {
        if (CollectionUtils.isEmpty(channelContentIds)) {
            return new HashMap<>();
        }
        List<ChangwenArticleDatastat> result = changwenArticleDatastatRepository.findAllById(channelContentIds);
        if (CollectionUtils.isEmpty(result)) {
            return new HashMap<>();
        }
        return result.stream().collect(Collectors.toMap(ChangwenArticleDatastat::getArticleId, o -> o));
    }

}
